@@ -127,7 +127,6 @@ class Launcher(_Launcher):
         deprecated_reason=None,
         deprecated_replacement=None,
     ):
-
         super().add_command(
             name=name, module_name=module_name, description=description, command=command
         )
@@ -219,6 +218,11 @@ def main():
         description="convert spec-edf acquisition to nexus - hdf5"
         "format to nx compliant file format",
     )
+    launcher.add_command(
+        "edf-quick-start",
+        module_name="nxtomomill.app.edfquickstart",
+        description="create a configuration file to convert from spec-edf to NXtomo",
+    )
     launcher.add_command(
         "tomoh52nx",
         module_name="nxtomomill.app.h52nx",
@@ -233,6 +237,16 @@ def main():
         module_name="nxtomomill.app.h52nx",
         description="convert bliss hdf5 to nexus hdf5",
     )
+    launcher.add_command(
+        "z-concatenate-scans",
+        module_name="nxtomomill.app.z_concatenate_scans",
+        description="Build a concatenation nexus file from a z-series",
+    )
+    launcher.add_command(
+        "zstages2nxs",
+        module_name="nxtomomill.app.zstages2nxs",
+        description="""creates all the target NXTomos for all the stages. Possibly reducing also reference scans for obtaining flats/dark""",
+    )
     launcher.add_command(
         "h5-3dxrd-2nx",
         module_name="nxtomomill.app.h5_3dxrd2nx",
