@@ -58,6 +58,9 @@ Application to convert from dx file (HDF5) to NXTomo (HDF5) file
                             incident beam energy in keV
       --overwrite           Do not ask for user permission to overwrite output files
       --data-copy           Force data duplication of frames. This will permit to have an 'all-embed' file. Otherwise we will have link between the dxfile and the NXTomo.
+
+For a complete tutorial you can have a look at :ref:`dxfile2nxtutorial`
+
 """
 
 __authors__ = [
@@ -225,9 +228,3 @@ def main(argv):
         duplicate_data=options.duplicate_data,
         energy=energy,
     )
-
-
-if __name__ == "__main__":
-    import sys
-
-    main(sys.argv[1:])
