@@ -30,7 +30,7 @@ Application to convert a bliss-hdf5 tomography dataset to Nexus - NXtomo (hdf5)
 
     usage: nxtomomill h52nx [-h] [--file-extension FILE_EXTENSION] [--single-file] [--overwrite] [--debug] [--entries ENTRIES] [--ignore-sub-entries IGNORE_SUB_ENTRIES] [--raises-error]
                             [--field-of-view FIELD_OF_VIEW] [--no-input] [--data-copy] [--x_trans_keys X_TRANS_KEYS] [--y_trans_keys Y_TRANS_KEYS] [--z_trans_keys Z_TRANS_KEYS]
-                            [--sample-detector-distance SAMPLE_DETECTOR_DISTANCE] [--valid_camera_names VALID_CAMERA_NAMES] [--rot_angle_keys ROT_ANGLE_KEYS] [--acq_expo_time_keys ACQ_EXPO_TIME_KEYS]
+                            [--sample-detector-distance-paths SAMPLE_DETECTOR_DISTANCE_PATHS] [--valid_camera_names VALID_CAMERA_NAMES] [--rot_angle_keys ROT_ANGLE_KEYS] [--acq_expo_time_keys ACQ_EXPO_TIME_KEYS]
                             [--x_pixel_size_key X_PIXEL_SIZE_KEY] [--y_pixel_size_key Y_PIXEL_SIZE_KEY] [--init_titles INIT_TITLES] [--init_zserie_titles INIT_ZSERIE_TITLES] [--dark_titles DARK_TITLES]
                             [--ref_titles REF_TITLES] [--proj_titles PROJ_TITLES] [--align_titles ALIGN_TITLES] [--set-params [SET_PARAMS [SET_PARAMS ...]]] [--format FORMAT] [--config CONFIG]
                             [input_file] [output_file]
@@ -64,7 +64,7 @@ Application to convert a bliss-hdf5 tomography dataset to Nexus - NXtomo (hdf5)
                             y translation key in bliss HDF5 file
       --z_trans_keys Z_TRANS_KEYS, --z-trans-keys Z_TRANS_KEYS
                             z translation key in bliss HDF5 file
-      --sample-detector-distance SAMPLE_DETECTOR_DISTANCE, --distance SAMPLE_DETECTOR_DISTANCE
+      --sample-detector-distance-paths SAMPLE_DETECTOR_DISTANCE_PATHS, --distance SAMPLE_DETECTOR_DISTANCE_PATHS
                             sample detector distance
       --valid_camera_names VALID_CAMERA_NAMES, --valid-camera-names VALID_CAMERA_NAMES
                             Valid NXDetector dataset name to be considered. Otherwise willtry to deduce them from NX_class attibute (value should beNXdetector) or from instrument group child structure.
@@ -247,10 +247,10 @@ def main(argv):
         help="z translation key in bliss HDF5 file",
     )
     parser.add_argument(
-        "--sample-detector-distance",
-        "--distance",
+        "--sample-detector-distance-paths",
+        "--distance-paths",
         default=None,
-        help="sample detector distance",
+        help="sample detector distance paths",
     )
     parser.add_argument(
         "--valid_camera_names",
