@@ -1,6 +1,6 @@
 from .edf.edfconverter import edf_to_nx  # noqa F401
+from .edf.edfconverter import from_edf_to_nx  # noqa F401
 from .hdf5.hdf5converter import from_h5_to_nx  # noqa F401
-from .hdf5.hdf5converter import h5_to_nx  # noqa F401
 from .hdf5.hdf5converter import get_bliss_tomo_entries  # noqa F401
 from .edf.edfconverter import EDFFileKeys  # noqa F401
 from .dxfile.dxfileconverter import from_dx_to_nx  # noqa F401
