@@ -34,7 +34,11 @@ import numpy
 import tempfile
 import os
 from nxtomomill import converter
-from tomoscan.esrf.hdf5scan import HDF5TomoScan
+
+try:
+    from tomoscan.esrf.scan.hdf5scan import HDF5TomoScan
+except ImportError:
+    from tomoscan.esrf.hdf5scan import HDF5TomoScan
 from nxtomomill.test.utils.dxfile import MockDxFile
 from silx.io.utils import get_data
 from tomoscan.validator import is_valid_for_reconstruction
@@ -97,10 +101,10 @@ class TestDxToNxConverter(unittest.TestCase):
                 numpy.testing.assert_array_equal(
                     self.mock.data_proj[-1], get_data(scan.projections[idx_last_proj])
                 )
-                self.assertEqual(scan.rotation_angle[0], 0)
-                self.assertEqual(scan.rotation_angle[5], 0)
-                self.assertEqual(scan.rotation_angle[6], 0)
-                self.assertEqual(scan.rotation_angle[-1], 180)
+                self.assertEqual(scan.rotation_angle[0], 0)  # pylint: disable=E1136
+                self.assertEqual(scan.rotation_angle[5], 0)  # pylint: disable=E1136
+                self.assertEqual(scan.rotation_angle[6], 0)  # pylint: disable=E1136
+                self.assertEqual(scan.rotation_angle[-1], 180)  # pylint: disable=E1136
 
                 # if overwrite not requested should fail on reprocessing
                 with self.assertRaises(OSError):
