@@ -52,6 +52,7 @@ from tomoscan.nexus.paths.nxtomo import get_paths as get_nexus_paths
 from silx.utils.deprecation import deprecated
 from tomoscan.esrf.scan.utils import get_parameters_frm_par_or_info
 from tomoscan.unitsystem.energysystem import EnergySI
+from tomoscan.unitsystem.electriccurrentsystem import ElectricCurrentSystem
 from tomoscan.esrf.scan.edfscan import EDFTomoScan
 from tomoscan.io import HDF5File
 import fabio
@@ -403,13 +404,19 @@ def from_edf_to_nx(configuration: TomoEDFConfig, progress=None) -> tuple:
         )
         h5d["/entry/sample/rotation_angle"].attrs["units"] = "degree"
 
+        # machine electric current
         nexus_paths = get_nexus_paths(LATEST_VERSION)
-        electric_current_dataset = h5d.create_dataset(
-            "/".join(["entry", nexus_paths.ELECTRIC_CURRENT_PATH]),
+        machine_elec_current_path = "/".join(
+            ["entry", nexus_paths.ELECTRIC_CURRENT_PATH]
+        )
+        machine_electric_current_dataset = h5d.create_dataset(
+            machine_elec_current_path,
             shape=(n_frames,),
             dtype=numpy.float32,
         )
-        h5d["/entry/sample/rotation_angle"].attrs["units"] = "degree"
+        h5d[machine_elec_current_path].attrs["units"] = str(
+            ElectricCurrentSystem.AMPERE
+        )
 
         # provision for centering motors
         x_dataset = h5d.create_dataset(
@@ -510,13 +517,20 @@ def from_edf_to_nx(configuration: TomoEDFConfig, progress=None) -> tuple:
                     )
 
             if srcur_index == -1:
-                electric_current_dataset[nf] = default_current
+                machine_electric_current_dataset[nf] = (
+                    default_current * configuration.machine_elec_current_unit.value
+                )
             else:
                 try:
                     str_counter_val = header.get("counter_pos", "").split(" ")
-                    electric_current_dataset[nf] = float(str_counter_val[srcur_index])
+                    machine_electric_current_dataset[nf] = (
+                        float(str_counter_val[srcur_index])
+                        * configuration.machine_elec_current_unit.value
+                    )
                 except IndexError:
-                    electric_current_dataset[nf] = default_current
+                    machine_electric_current_dataset[nf] = (
+                        default_current * configuration.machine_elec_current_unit.value
+                    )
 
             nf += 1
             if progress is not None:
@@ -608,15 +622,22 @@ def from_edf_to_nx(configuration: TomoEDFConfig, progress=None) -> tuple:
                                 * configuration.z_trans_unit.value
                             )
                     if srcur_index == -1:
-                        electric_current_dataset[nfr] = default_current
+                        machine_electric_current_dataset[nfr] = (
+                            default_current
+                            * configuration.machine_elec_current_unit.value
+                        )
                     else:
                         str_counter_val = header.get("counter_pos", "").split(" ")
                         try:
-                            electric_current_dataset[nfr] = float(
-                                str_counter_val[srcur_index]
+                            machine_electric_current_dataset[nfr] = (
+                                float(str_counter_val[srcur_index])
+                                * configuration.machine_elec_current_unit.value
                             )
                         except IndexError:
-                            electric_current_dataset[nfr] = default_current
+                            machine_electric_current_dataset[nfr] = (
+                                default_current
+                                * configuration.machine_elec_current_unit.value
+                            )
 
                     nfr += 1
 
@@ -739,13 +760,20 @@ def from_edf_to_nx(configuration: TomoEDFConfig, progress=None) -> tuple:
                         * configuration.z_trans_unit.value
                     )
             if srcur_index == -1:
-                electric_current_dataset[nf] = default_current
+                machine_electric_current_dataset[nf] = (
+                    default_current * configuration.machine_elec_current_unit.value
+                )
             else:
                 try:
                     str_counter_val = header.get("counter_pos", "").split(" ")
-                    electric_current_dataset[nf] = float(str_counter_val[srcur_index])
+                    machine_electric_current_dataset[nf] = (
+                        float(str_counter_val[srcur_index])
+                        * configuration.machine_elec_current_unit.value
+                    )
                 except IndexError:
-                    electric_current_dataset[nf] = default_current
+                    machine_electric_current_dataset[nf] = (
+                        default_current * configuration.machine_elec_current_unit.value
+                    )
             nf += 1
             nproj += 1
 
