@@ -100,13 +100,16 @@ class BaseAcquisition:
 
     _END_TIME_PATH = ("end_time",)
 
-    _TECHNIQUE_MOTOR_PATH = "technique/scan/motor"
+    _TECHNIQUE_MOTOR_PATHS = ("technique/scan/motor", "technique/proj/motor")
 
     _SOURCE_NAME = ("instrument/machine/name",)
 
     _SOURCE_TYPE = ("instrument/machine/type",)
 
-    _FRAME_FLIP_PATH = "technique/detector/flipping"
+    _FRAME_FLIP_PATHS = (
+        "technique/detector/{detector_name}/flipping",
+        "technique/detector/flipping",
+    )
 
     def __init__(
         self,
@@ -261,7 +264,7 @@ class BaseAcquisition:
                 nx_tomo=nx_tomo,
             )
 
-    def to_NXtomos(self, request_input, input_callback) -> tuple:
+    def to_NXtomos(self, request_input, input_callback, check_tomo_n=True) -> tuple:
         raise NotImplementedError("Base class")
 
     @property
@@ -447,25 +450,26 @@ class BaseAcquisition:
             return None
         else:
             with EntryReader(self._root_url) as entry:
-                if self._TECHNIQUE_MOTOR_PATH in entry:
-                    try:
-                        rotation_motor = get_dataset_name_from_motor(
-                            motors=h5py_read_dataset(
-                                numpy.asarray(entry[self._TECHNIQUE_MOTOR_PATH])
-                            ),
-                            motor_name="rotation",
-                        )
-                    except Exception as e:
-                        _logger.error(e)
+                for motor_path in self._TECHNIQUE_MOTOR_PATHS:
+                    if motor_path in entry:
+                        try:
+                            rotation_motor = get_dataset_name_from_motor(
+                                motors=h5py_read_dataset(
+                                    numpy.asarray(entry[motor_path])
+                                ),
+                                motor_name="rotation",
+                            )
+                        except Exception as e:
+                            _logger.error(e)
+                        else:
+                            return rotation_motor
                     else:
-                        return rotation_motor
-                else:
-                    _logger.warning(
-                        "{} unable to find rotation motor from {}".format(
-                            self._TECHNIQUE_MOTOR_PATH, self._root_url
+                        _logger.warning(
+                            "{} unable to find rotation motor from {}".format(
+                                motor_path, self._root_url
+                            )
                         )
-                    )
-                    return None
+                return None
 
     def _get_electric_current(self, root_node) -> list:
         """retrieve electric current provide a time stamp for each of them"""
@@ -497,7 +501,7 @@ class BaseAcquisition:
                     pass
                 else:
                     # handle case where elec_current is a scalar. Cast it to list before return
-                    if isinstance(elec_current, numpy.number):
+                    if numpy.isscalar(elec_current):
                         elec_current = [
                             elec_current,
                         ]
