@@ -34,11 +34,14 @@ __license__ = "MIT"
 __date__ = "14/02/2022"
 
 
+from datetime import datetime
+
+from nxtomomill.utils.utils import str_datetime_to_numpy_datetime64
 from .baseacquisition import BaseAcquisition
 from nxtomomill.nexus.nxsource import SourceType
 from nxtomomill.io.acquisitionstep import AcquisitionStep
 from .baseacquisition import EntryReader
-from .utils import get_entry_type
+from .utils import deduce_machine_electric_current, get_entry_type
 from .utils import guess_nx_detector
 from .utils import get_nx_detectors
 from nxtomomill.utils import ImageKey
@@ -47,6 +50,7 @@ from silx.io.utils import h5py_read_dataset
 import h5py
 from silx.io.url import DataUrl
 from typing import Optional, Union
+from tomoscan.unitsystem import ElectricCurrentSystem
 
 try:
     import hdf5plugin  # noqa F401
@@ -82,6 +86,7 @@ class StandardAcquisition(BaseAcquisition):
         configuration: TomoHDF5Config,
         detector_sel_callback,
         start_index,
+        parent=None,
     ):
         super().__init__(
             root_url=root_url,
@@ -89,6 +94,8 @@ class StandardAcquisition(BaseAcquisition):
             detector_sel_callback=detector_sel_callback,
             start_index=start_index,
         )
+        self._parent = parent
+        # possible parent. Like for z series
         self._nx_tomos = [NXtomo("/")]
         self._image_key_control = None
         self._rotation_angle = None
@@ -98,16 +105,26 @@ class StandardAcquisition(BaseAcquisition):
         self._y_translation = None
         """y_translation"""
         self._z_translation = None
+        self._x_flipped = None
+        self._y_flipped = None
+
+        self._unique_detector_names = list()
+        # register names
 
-        # self._n_frames = None
-        # self._dim_1 = None
-        # self._dim_2 = None
-        # self._data_type = None
         self._virtual_sources = None
         self._acq_expo_time = None
         self._copied_dataset = {}
         "register dataset copied. Key if the original location as" "DataUrl.path. Value is the DataUrl it has been moved to"
-        # self._current_scan_n_frame = None
+        self._known_machine_electric_current = None
+        # store all registred amchine electric current
+        self._frames_timestamp = None
+        # try to deduce time stamp of each frame
+
+    def parent_root_url(self) -> Optional[DataUrl]:
+        if self._parent is not None:
+            return self._parent.root_url
+        else:
+            return None
 
     def get_expected_nx_tomo(self):
         return 1
@@ -132,10 +149,22 @@ class StandardAcquisition(BaseAcquisition):
     def z_translation(self):
         return self._z_translation
 
+    @property
+    def x_flipped(self):
+        return self._x_flipped
+
+    @property
+    def y_flipped(self):
+        return self._y_flipped
+
     @property
     def n_frames(self):
         return self._n_frames
 
+    @property
+    def n_frames_actual_bliss_scan(self):
+        return self._n_frames_actual_bliss_scan
+
     @property
     def dim_1(self):
         return self._dim_1
@@ -152,6 +181,13 @@ class StandardAcquisition(BaseAcquisition):
     def expo_time(self):
         return self._acq_expo_time
 
+    @property
+    def known_machine_electric_current(self) -> Optional[dict]:
+        """
+        Return the dict of all know machine electric current. Key is the time stamp, value is the electric current
+        """
+        return self._known_machine_electric_current
+
     @property
     def is_xrd_ct(self):
         return False
@@ -251,14 +287,13 @@ class StandardAcquisition(BaseAcquisition):
         if data_dataset.ndim == 2:
             shape = (1, data_dataset.shape[0], data_dataset.shape[1])
         elif data_dataset.ndim != 3:
-            err = "dataset %s is expected to be 3D when %sD found." % (
-                data_name,
-                data_dataset.ndim,
-            )
+            err = f"dataset {data_name} is expected to be 3D when {data_dataset.ndim}D found."
             if data_dataset.ndim == 1:
                 err = "\n".join(
-                    err,
-                    "This might be a bliss-EDF dataset. Those are not handled by nxtomomill",
+                    [
+                        err,
+                        "This might be a bliss-EDF dataset. Those are not handled by nxtomomill",
+                    ]
                 )
             raise ValueError(err)
         else:
@@ -266,6 +301,7 @@ class StandardAcquisition(BaseAcquisition):
 
         n_frame = shape[0]
         self._n_frames += n_frame
+        self._n_frames_actual_bliss_scan = n_frame
         if self.dim_1 is None:
             self._dim_2 = shape[1]
             self._dim_1 = shape[2]
@@ -289,7 +325,10 @@ class StandardAcquisition(BaseAcquisition):
         # "treated file". Because .name if the name on the 'target'
         # file of the virtual dataset
         v_source = h5py.VirtualSource(
-            camera_dataset_url.file_path(), data_dataset_path, data_dataset.shape
+            camera_dataset_url.file_path(),
+            data_dataset_path,
+            data_dataset.shape,
+            dtype=self._data_type,
         )
         self._virtual_sources.append(v_source)
         self._virtual_sources_len.append(n_frame)
@@ -303,11 +342,13 @@ class StandardAcquisition(BaseAcquisition):
         input_file_path,
         entry_path,
         entry_url,
-    ):
+    ) -> bool:
+        """
+        return True if the entry contains frames
+        """
         if "data_cast" in detector_node:
             _logger.warning(
-                "!!! looks like this data has been cast. Take cast data for %s!!!"
-                % detector_node
+                f"!!! looks like this data has been cast. Take cast data for {detector_node}!!!"
             )
             data_dataset = detector_node["data_cast"]
             data_name = "/".join((detector_node.name, "data_cast"))
@@ -332,6 +373,14 @@ class StandardAcquisition(BaseAcquisition):
             entry_url.path()
         ]
 
+        x_flipped, y_flipped = self._get_flipped_frame()
+        if x_flipped is not None and y_flipped is not None:
+            if self._x_flipped is None and self._y_flipped is None:
+                self._x_flipped, self._y_flipped = bool(x_flipped), bool(y_flipped)
+            elif x_flipped != self._x_flipped or y_flipped != self._y_flipped:
+                raise ValueError(
+                    f"Found different detector flips inside the same sequence on {entry}. Unable to handle it."
+                )
         # store rotation
         rots = self._get_rotation_angle(root_node=entry, n_frame=n_frame)[0]
         self._rotation_angle.extend(rots)
@@ -411,8 +460,7 @@ class StandardAcquisition(BaseAcquisition):
 
             if "instrument" not in entry:
                 _logger.error(
-                    "no instrument group found in %s, unable to"
-                    "retrieve frames" % entry.name
+                    f"no instrument group found in {entry.name}, unable to retrieve frames"
                 )
                 return
 
@@ -423,14 +471,13 @@ class StandardAcquisition(BaseAcquisition):
                 # update valid camera names
                 self.configuration.valid_camera_names = det_grps
 
-            for key, det_grp in instrument_grp.items():
+            has_frames = False
+            for key, _ in instrument_grp.items():
                 if (
                     "NX_class" in instrument_grp[key].attrs
                     and instrument_grp[key].attrs["NX_class"] == "NXdetector"
                 ):
-                    _logger.debug(
-                        "Found one detector at %s for %s." % (key, entry.name)
-                    )
+                    _logger.debug(f"Found one detector at {key} for {entry.name}.")
 
                     # diode
                     if self.has_diode:
@@ -449,7 +496,8 @@ class StandardAcquisition(BaseAcquisition):
                         continue
                     else:
                         detector_node = instrument_grp[key]
-                        _logger.info(f"start treatment of detector {detector_node}")
+                        if key not in self._unique_detector_names:
+                            self._unique_detector_names.append(key)
                         self._treate_valid_camera(
                             detector_node,
                             entry=entry,
@@ -458,11 +506,126 @@ class StandardAcquisition(BaseAcquisition):
                             entry_path=entry_path,
                             entry_url=entry_url,
                         )
+                        has_frames = True
+            # try to get some other metadata
+
+            # handle frame time stamp
+            start_time = self._get_start_time(entry)
+            if start_time is not None:
+                start_time = datetime.fromisoformat(start_time)
+            end_time = self._get_end_time(entry)
+            if end_time is not None:
+                end_time = datetime.fromisoformat(end_time)
+            if has_frames:
+                self._register_frame_timestamp(entry, start_time, end_time)
+
+            # handle electric current. Can retrieve some current even on bliss scan entry doesn;t containing directly frames
+            self._register_machine_electric_current(entry, start_time, end_time)
+
+    def _register_machine_electric_current(
+        self, entry: h5py.Group, start_time, end_time
+    ):
+        """Update machine electric current for provided entry (bliss scan"""
+        (
+            electric_currents,
+            electric_current_unit,
+        ) = self._get_electric_current(root_node=entry)
+        electric_current_unit_ref = ElectricCurrentSystem.AMPERE
+        # electric current will be saved as Ampere
+        if electric_currents is not None and len(electric_currents) > 0:
+            if electric_current_unit is None:
+                electric_current_unit = ElectricCurrentSystem.MILLIAMPERE
+                _logger.warning(
+                    "No unit found for electric current. Consider it as mA."
+                )
+
+            unit_factor = (
+                ElectricCurrentSystem.from_str(electric_current_unit).value
+                / electric_current_unit_ref.value
+            )
+
+            new_know_electric_currents = {}
+            if start_time is None or end_time is None:
+                if start_time != end_time:
+                    _logger.warning(
+                        f"Unable to find {'start_time' if start_time is None else 'end_time'}. Will pick the first available electric_current for the frame"
+                    )
+                    t_time = start_time or end_time
+                    # if at least one can find out
+                    new_know_electric_currents[
+                        str_datetime_to_numpy_datetime64(t_time)
+                    ] = (electric_currents[0] * unit_factor)
+                else:
+                    _logger.error(
+                        "Unable to find start_time and end_time. Will not register any machine electric current"
+                    )
+            elif len(electric_currents) == 1:
+                # if we have only one value, consider the machine electric current is constant during this time
+                # might be improved later if we can know if current is determine at the
+                # beginning or the end. But should have no impact
+                # as the time slot is short
+                new_know_electric_currents[
+                    str_datetime_to_numpy_datetime64(start_time)
+                ] = (electric_currents[0] * unit_factor)
+            else:
+                # linspace from datetime within ms precision.
+                # see https://localcoder.org/creating-numpy-linspace-out-of-datetime#credit_4
+                # and https://stackoverflow.com/questions/37964100/creating-numpy-linspace-out-of-datetime
+                timestamps = numpy.linspace(
+                    start=str_datetime_to_numpy_datetime64(start_time).astype(
+                        numpy.float128
+                    ),
+                    stop=str_datetime_to_numpy_datetime64(end_time).astype(
+                        numpy.float128
+                    ),
+                    num=len(electric_currents),
+                    endpoint=True,
+                    dtype="<M8[ms]",
+                )
+                for timestamp, mach_electric_current in zip(
+                    timestamps, electric_currents
+                ):
+                    new_know_electric_currents[timestamp.astype(numpy.datetime64)] = (
+                        mach_electric_current * unit_factor
+                    )
+            self._known_machine_electric_current.update(new_know_electric_currents)
+
+    def _register_frame_timestamp(self, entry: h5py.Group, start_time, end_time):
+        """
+        update frame time stamp for the provided entry (bliss scan)
+        """
+        if start_time is None or end_time is None:
+            if start_time != end_time:
+                t_time = str_datetime_to_numpy_datetime64(start_time or end_time)
+                message = f"Unable to find start_time and / or end_time. Takes {t_time} as frame time stamp for {entry} "
+                self._frames_timestamp.extend(
+                    [t_time] * self._n_frames_actual_bliss_scan
+                )
+                _logger.warning(message)
+            else:
+                message = f"Unable to find start_time and end_time. Can't deduce frames time stamp for {entry}"
+                _logger.error(message)
+        else:
+            frames_times_stamps_as_f8 = numpy.linspace(
+                start=str_datetime_to_numpy_datetime64(start_time).astype(
+                    numpy.float128
+                ),
+                stop=str_datetime_to_numpy_datetime64(end_time).astype(numpy.float128),
+                num=self._n_frames_actual_bliss_scan,
+                endpoint=True,
+                dtype="<M8[ms]",
+            )
+            frames_times_stamps_as_f8 = [
+                timestamp.astype("<M8[ms]") for timestamp in frames_times_stamps_as_f8
+            ]
+            self._frames_timestamp.extend(frames_times_stamps_as_f8)
 
     def _preprocess_registered_entries(self):
         """parse all frames of the different steps and retrieve data,
         image_key..."""
         self._n_frames = 0
+        self._n_frames_actual_bliss_scan = 0
+        # number of frame contains in X.1
         self._dim_1 = None
         self._dim_2 = None
         self._data_type = None
@@ -471,12 +634,16 @@ class StandardAcquisition(BaseAcquisition):
         self._z_translation = []
         self._image_key_control = []
         self._rotation_angle = []
+        self._known_machine_electric_current = {}
+        self._frames_timestamp = []
         self._virtual_sources = []
         self._virtual_sources_len = []
         self._diode = []
         self._acq_expo_time = []
         self._diode_unit = None
         self._copied_dataset = {}
+        self._x_flipped = None
+        self._y_flipped = None
 
         # if rotation motor is not defined try to deduce it from root_url/technique/scan/motor
         if self.configuration.rotation_angle_keys is None:
@@ -530,43 +697,55 @@ class StandardAcquisition(BaseAcquisition):
                 res[alias] = self.configuration.param_already_defined[alias]
         return res
 
-    def _generic_path_getter(self, path, message, level="warning"):
+    def _generic_path_getter(self, paths: tuple, message, level="warning", entry=None):
         """
         :param str level: level can be logging.level values : "warning", "error", "info"
+        :param H5group entry: user can provide directly an entry to be used as an open h5Group
         """
-        if self.root_url is None:
-            return None
-        self._check_has_metadata()
-        with self.read_entry() as entry:
-            if path in entry:
-                return h5py_read_dataset(entry[path])
-            else:
-                if message is not None:
-                    getattr(_logger, level)(message)
+        if not isinstance(paths, tuple):
+            raise TypeError
+
+        url = self.parent_root_url() or self.root_url
+        if url is not None:
+            self._check_has_metadata(url)
+
+        def process(h5_group):
+            for path in paths:
+                if h5_group is not None and path in h5_group:
+                    return h5py_read_dataset(h5_group[path])
+            if message is not None:
+                getattr(_logger, level)(message)
+
+        if entry is None:
+            if url is None:
                 return None
+            with EntryReader(url) as h5_group:
+                return process(h5_group)
+        else:
+            return process(entry)
 
     def _get_source_name(self):
         """ """
         return self._generic_path_getter(
-            path=self._SOURCE_NAME, message="Unable to find source name", level="info"
+            paths=self._SOURCE_NAME, message="Unable to find source name", level="info"
         )
 
     def _get_source_type(self):
         """ """
         return self._generic_path_getter(
-            path=self._SOURCE_TYPE, message="Unable to find source type", level="info"
+            paths=self._SOURCE_TYPE, message="Unable to find source type", level="info"
         )
 
     def _get_title(self):
         """return acquisition title"""
         return self._generic_path_getter(
-            path=self._TITLE_PATH, message="Unable to find title"
+            paths=self._TITLE_PATH, message="Unable to find title"
         )
 
     def _get_instrument_name(self):
         """:return instrument title / name"""
         return self._generic_path_getter(
-            path=self._INSTRUMENT_NAME_PATH,
+            paths=self._INSTRUMENT_NAME_PATH,
             message="Unable to find instrument name",
             level="info",
         )
@@ -574,7 +753,7 @@ class StandardAcquisition(BaseAcquisition):
     def _get_dataset_name(self):
         """return name of the acquisition"""
         return self._generic_path_getter(
-            path=self._DATASET_NAME_PATH,
+            paths=self._DATASET_NAME_PATH,
             message="No name describing the acquisition has been "
             "found, Name dataset will be skip",
         )
@@ -582,7 +761,7 @@ class StandardAcquisition(BaseAcquisition):
     def _get_sample_name(self):
         """return sample name"""
         return self._generic_path_getter(
-            path=self._SAMPLE_NAME_PATH,
+            paths=self._SAMPLE_NAME_PATH,
             message="No sample name has been "
             "found, Sample name dataset will be skip",
         )
@@ -591,34 +770,55 @@ class StandardAcquisition(BaseAcquisition):
         """return the nb_scans composing the zseries if is part of a group
         of sequence"""
         return self._generic_path_getter(
-            path=self._GRP_SIZE_PATH,
+            paths=self._GRP_SIZE_PATH,
             message=None,
         )
 
     def _get_tomo_n(self):
         return self._generic_path_getter(
-            path=self._TOMO_N_PATH,
+            paths=self._TOMO_N_PATH,
             message="unable to find information regarding tomo_n",
         )
 
-    def _get_start_time(self):
+    def _get_start_time(self, entry=None):
         return self._generic_path_getter(
-            path=self._START_TIME_PATH,
+            paths=self._START_TIME_PATH,
             message="Unable to find start time",
             level="info",
+            entry=entry,
         )
 
-    def _get_end_time(self):
+    def _get_end_time(self, entry=None):
         return self._generic_path_getter(
-            path=self._END_TIME_PATH, message="Unable to find end time", level="info"
+            paths=self._END_TIME_PATH,
+            message="Unable to find end time",
+            level="info",
+            entry=entry,
         )
 
+    def _get_flipped_frame(self):
+        url = self.parent_root_url() or self.root_url
+        if url is None:
+            return None, None
+        self._check_has_metadata(url)
+        with EntryReader(url) as entry:
+            for flip_path in self._FRAME_FLIP_PATHS:
+                if len(self._unique_detector_names) > 0:
+                    key = flip_path.format(detector_name=self._unique_detector_names[0])
+                else:
+                    key = flip_path
+                if key in entry:
+                    return h5py_read_dataset(entry[key])
+            else:
+                return None, None
+
     def _get_energy(self, ask_if_0, input_callback):
         """return tuple(energy, unit)"""
-        if self.root_url is None:
+        url = self.parent_root_url() or self.root_url
+        if url is None:
             return None, None
         self._check_has_metadata()
-        with self.read_entry() as entry:
+        with EntryReader(url) as entry:
             if self._ENERGY_PATH in entry:
                 energy = h5py_read_dataset(entry[self._ENERGY_PATH])
                 unit = self._get_unit(entry[self._ENERGY_PATH], default_unit="kev")
@@ -645,10 +845,11 @@ class StandardAcquisition(BaseAcquisition):
 
     def _get_distance(self):
         """return tuple(distance, unit)"""
-        if self.root_url is None:
+        url = self.parent_root_url() or self.root_url
+        if url is None:
             return None, None
-        self._check_has_metadata()
-        with self.read_entry() as entry:
+        self._check_has_metadata(url)
+        with EntryReader(url) as entry:
             for key in self.configuration.sample_detector_distance_paths:
                 if key in entry:
                     node = entry[key]
@@ -667,18 +868,72 @@ class StandardAcquisition(BaseAcquisition):
                 _logger.warning(mess)
                 return 1.0, "m"
 
+    def _get_real_pixel_size(self, axis):
+        url = self.parent_root_url() or self.root_url
+        if url is None:
+            return None, None
+        if axis == "x":
+            keys = self.configuration.x_real_pixel_size_paths
+        elif axis == "y":
+            keys = self.configuration.y_real_pixel_size_paths
+        else:
+            raise ValueError(f"axis {axis} is invalid")
+
+        with EntryReader(url) as entry:
+            for key in keys:
+                if len(self._unique_detector_names) == 0 and "{detector_name}" in key:
+                    continue
+                else:
+                    if (
+                        len(self._unique_detector_names) > 1
+                        and "{detector_name}" in key
+                    ):
+                        _logger.warning(
+                            f"more than one detector found. Can only handle one ({self._unique_detector_names[0]})"
+                        )
+                    if len(self._unique_detector_names) > 0:
+                        path = key.format(detector_name=self._unique_detector_names[0])
+                    else:
+                        path = key
+                if path in entry:
+                    node = entry[path]
+                    node_item = h5py_read_dataset(node)
+                    # if the pixel size is provided as x, y
+                    if isinstance(node_item, numpy.ndarray):
+                        if len(node_item) > 1 and axis == "y":
+                            size_ = node_item[1]
+                        else:
+                            size_ = node_item[0]
+                    # if this is a single value
+                    else:
+                        size_ = node_item
+                    unit = self._get_unit(node, default_unit="micrometer")
+                    # convert to meter
+                    size_ = size_ * metricsystem.MetricSystem.from_value(unit).value
+                    return size_, "m"
+
+            mess = "unable to find {} real pixel size for entry {}".format(axis, entry)
+            mess += "default value will be set to 10-6m"
+            _logger.warning(mess)
+            return 10e-6, "m"
+
     def _get_pixel_size(self, axis):
         """return tuple(pixel_size, unit)"""
-        if self.root_url is None:
+        url = self.parent_root_url() or self.root_url
+        if url is None:
             return None, None
-        assert axis in ("x", "y")
+        if axis not in ("x", "y"):
+            raise ValueError
         self._check_has_metadata()
-        keys = (
-            self.configuration.x_pixel_size_paths
-            if axis == "x"
-            else self.configuration.y_pixel_size_paths
-        )
-        with self.read_entry() as entry:
+
+        if axis == "x":
+            keys = self.configuration.x_pixel_size_paths
+        elif axis == "y":
+            keys = self.configuration.y_pixel_size_paths
+        else:
+            raise ValueError(f"axis {axis} is invalid")
+
+        with EntryReader(url) as entry:
             for key in keys:
                 if key in entry:
                     node = entry[key]
@@ -705,22 +960,36 @@ class StandardAcquisition(BaseAcquisition):
                 _logger.warning(mess)
                 return 10e-6, "m"
 
+    def _get_magnification(self):
+        keys = self.configuration.magnification_keys
+        # for z series this information is in the parent root url
+        url = self.parent_root_url() or self.root_url
+        if url is None:
+            return None
+        with EntryReader(url) as entry:
+            for key in keys:
+                if key in entry:
+                    return h5py_read_dataset(entry[key])
+            else:
+                _logger.warning(
+                    f"unable to find information regarding magnification for entry {entry}"
+                )
+                return None
+
     def _get_field_of_fiew(self):
         if self.configuration.field_of_view is not None:
             return self.configuration.field_of_view.value
-        if self.root_url is None:
+        url = self.parent_root_url() or self.root_url
+        if url is None:
             return None
-        with self.read_entry() as entry:
+        with EntryReader(url) as entry:
             if self._FOV_PATH in entry:
                 return h5py_read_dataset(entry[self._FOV_PATH])
             else:
-                mess = (
-                    "unable to find information regarding field of view for"
-                    " entry {}".format(entry)
+                # FOV is optional: don't raise an error
+                _logger.warning(
+                    f"unable to find information regarding field of view for entry {entry}. set it to default value (Full)"
                 )
-                # FOV is optional: shouldn't won't raise an error
-                mess += "set it to default value (Full)"
-                _logger.warning(mess)
                 return "Full"
 
     def _get_estimated_cor_from_motor(self, pixel_size):
@@ -757,7 +1026,7 @@ class StandardAcquisition(BaseAcquisition):
     def to_NXtomos(self, request_input, input_callback, check_tomo_n=True) -> tuple:
         self._preprocess_registered_entries()
 
-        nx_tomo = NXtomo("/")
+        nx_tomo = NXtomo()
 
         # 1. root level information
         # start and end time
@@ -770,13 +1039,20 @@ class StandardAcquisition(BaseAcquisition):
         nx_tomo.group_size = self._get_grp_size()
 
         # 2. define beam
-        energy, unit = self._get_user_settable_parameter(
-            param_key=TomoHDF5Config.EXTRA_PARAMS_ENERGY_DK,
-            fallback_fct=self._get_energy,
-            dtype=float,
-            input_callback=input_callback,
-            ask_if_0=request_input,
-        )
+        try:
+            energy, unit = self._get_user_settable_parameter(
+                param_key=TomoHDF5Config.EXTRA_PARAMS_ENERGY_DK,
+                fallback_fct=self._get_energy,
+                dtype=float,
+                input_callback=input_callback,
+                ask_if_0=request_input,
+            )
+        except TypeError:
+            # if the path lead to unexpected dataset (group instead of a dataset or a broken folder)
+            _logger.error("Fail to get energy")
+            energy = None
+            unit = None
+
         if energy is not None:
             # TODO: better manamgent of energy ? might be energy.beam or energy.instrument.beam ?
             nx_tomo.energy = energy
@@ -788,36 +1064,75 @@ class StandardAcquisition(BaseAcquisition):
         nx_tomo.instrument.detector.count_time = self._acq_expo_time
         nx_tomo.instrument.detector.count_time.unit = "s"
         # distance
-        distance, unit = self._get_user_settable_parameter(
-            param_key=TomoHDF5Config.EXTRA_PARAMS_DISTANCE,
-            fallback_fct=self._get_distance,
-            dtype=float,
-        )
+        try:
+            distance, unit = self._get_user_settable_parameter(
+                param_key=TomoHDF5Config.EXTRA_PARAMS_DISTANCE,
+                fallback_fct=self._get_distance,
+                dtype=float,
+            )
+        except TypeError:
+            # if the path lead to unexpected dataset (group instead of a dataset or a broken folder)
+            _logger.error("Fail to get sample/detector distance")
+            distance = None
+            unit = None
         if distance is not None:
             nx_tomo.instrument.detector.distance = distance
             if nx_tomo.instrument.detector.distance is not None:
                 nx_tomo.instrument.detector.distance.unit = unit
         # x and y pixel size
-        x_pixel_size, unit = self._get_user_settable_parameter(
-            param_key=TomoHDF5Config.EXTRA_PARAMS_X_PIXEL_SIZE_DK,
-            fallback_fct=self._get_pixel_size,
-            dtype=float,
-            axis="x",
-        )
-        nx_tomo.instrument.detector.x_pixel_size = x_pixel_size
+        try:
+            x_pixel_size, unit = self._get_user_settable_parameter(
+                param_key=TomoHDF5Config.EXTRA_PARAMS_X_PIXEL_SIZE_DK,
+                fallback_fct=self._get_pixel_size,
+                dtype=float,
+                axis="x",
+            )
+        except TypeError:
+            # if the path lead to unexpected dataset (group instead of a dataset or a broken folder)
+            _logger.error("Fail to get x_pixel_size")
+            unit = None
+            x_pixel_size = None
+        else:
+            nx_tomo.instrument.detector.x_pixel_size = x_pixel_size
         if unit is not None:
             nx_tomo.instrument.detector.x_pixel_size.unit = unit
 
-        y_pixel_size, unit = self._get_user_settable_parameter(
-            param_key=TomoHDF5Config.EXTRA_PARAMS_Y_PIXEL_SIZE_DK,
-            fallback_fct=self._get_pixel_size,
-            dtype=float,
-            axis="y",
-        )
-        nx_tomo.instrument.detector.y_pixel_size = y_pixel_size
+        try:
+            y_pixel_size, unit = self._get_user_settable_parameter(
+                param_key=TomoHDF5Config.EXTRA_PARAMS_Y_PIXEL_SIZE_DK,
+                fallback_fct=self._get_pixel_size,
+                dtype=float,
+                axis="y",
+            )
+        except TypeError:
+            # if the path lead to unexpected dataset (group instead of a dataset or a broken folder)
+            _logger.error("Fail to get y_pixel_size")
+            unit = None
+            y_pixel_size = None
+        else:
+            nx_tomo.instrument.detector.y_pixel_size = y_pixel_size
         if unit is not None:
             nx_tomo.instrument.detector.y_pixel_size.unit = unit
 
+        # x and y real size
+        nx_tomo.instrument.detector.x_real_pixel_size, unit = self._get_real_pixel_size(
+            axis="x"
+        )
+        if unit is not None:
+            nx_tomo.instrument.detector.x_real_pixel_size.unit = unit
+        nx_tomo.instrument.detector.y_real_pixel_size, unit = self._get_real_pixel_size(
+            axis="y"
+        )
+        if unit is not None:
+            nx_tomo.instrument.detector.y_real_pixel_size.unit = unit
+
+        # magnification
+        nx_tomo.instrument.detector.magnification = self._get_magnification()
+        # flips
+        nx_tomo.instrument.detector.x_flipped = self.x_flipped
+        nx_tomo.instrument.detector.y_flipped = self.y_flipped
+
+        # fov
         fov = self._get_field_of_fiew()
         if fov is not None:
             nx_tomo.instrument.detector.field_of_view = fov
@@ -831,7 +1146,7 @@ class StandardAcquisition(BaseAcquisition):
         # define tomo_n
         nx_tomo.instrument.detector.tomo_n = self._get_tomo_n()
 
-        # 3. define nx source
+        # 4. define nx source
         source_name = self._get_source_name()
         nx_tomo.instrument.source.name = source_name
         source_type = self._get_source_type()
@@ -846,7 +1161,7 @@ class StandardAcquisition(BaseAcquisition):
 
         nx_tomo.instrument.source.type = source_type
 
-        # 4. define sample
+        # 5. define sample
         nx_tomo.sample.name = self._get_sample_name()
         nx_tomo.sample.rotation_angle = self.rotation_angle
         nx_tomo.sample.x_translation.value = self.x_translation
@@ -856,7 +1171,21 @@ class StandardAcquisition(BaseAcquisition):
         nx_tomo.sample.z_translation.value = self.z_translation
         nx_tomo.sample.z_translation.unit = "m"
 
-        # 5. define diode
+        # 6. define control
+        if (
+            self.configuration.handle_machine_current
+            and self.known_machine_electric_current not in (None, dict())
+        ):
+            nx_tomo.control.data = deduce_machine_electric_current(
+                timestamps=self._frames_timestamp,
+                known_machine_electric_current=self._known_machine_electric_current,
+            )
+            nx_tomo.control.data.unit = ElectricCurrentSystem.AMPERE
+            types = set()
+            for d in nx_tomo.control.data.value:
+                types.add(type(d))
+
+        # 7. define diode
         if self.has_diode:
             nx_tomo.instrument.diode.data = self._diode
             nx_tomo.instrument.diode.data.unit = self._diode_unit
@@ -883,8 +1212,9 @@ class StandardAcquisition(BaseAcquisition):
                 else:
                     _logger.error(mess)
 
-    def _check_has_metadata(self):
-        if self.root_url is None:
+    def _check_has_metadata(self, url: Optional[DataUrl] = None):
+        url = url or self.root_url
+        if url is None:
             raise ValueError(
                 "no initialization entry specify, unable to" "retrieve energy"
             )
