@@ -82,6 +82,8 @@ H5_ALIGNMENT_TITLES = Tomo.H5.ALIGNMENT_TITLES
 H5_ACQ_EXPO_TIME_KEYS = Tomo.H5.ACQ_EXPO_TIME_KEYS
 H5_X_PIXEL_SIZE = Tomo.H5.X_PIXEL_SIZE
 H5_Y_PIXEL_SIZE = Tomo.H5.Y_PIXEL_SIZE
+H5_X_REAL_PIXEL_SIZE = Tomo.H5.X_REAL_PIXEL_SIZE
+H5_Y_REAL_PIXEL_SIZE = Tomo.H5.Y_REAL_PIXEL_SIZE
 H5_DARK_TITLES = Tomo.H5.DARK_TITLES
 H5_INIT_TITLES = Tomo.H5.INIT_TITLES
 H5_PCOTOMO_INIT_TITLES = Tomo.H5.PCOTOMO_INIT_TITLES
@@ -117,6 +119,8 @@ DEFAULT_H5_KEYS = H5FileKeys(
     H5_Y_PIXEL_SIZE,
     H5_DIODE_KEYS,
     H5_MAGNIFICATION_KEYS,
+    H5_X_REAL_PIXEL_SIZE,
+    H5_Y_REAL_PIXEL_SIZE,
 )
 
 
@@ -665,6 +669,7 @@ class _H5ToNxConverter(BaseConverter):
                 if self.configuration.raises_error:
                     raise e
                 else:
+                    raise e
                     _logger.error(
                         f"Fails to write {str(acquisition.root_url)}. Error is {str(e)}"
                     )
@@ -676,7 +681,7 @@ class _H5ToNxConverter(BaseConverter):
         # if we created one file per entry then create a master file with link to those entries
         if self.configuration.single_file is False and divide_into_sub_files:
             _logger.info(f"create link in {self.configuration.output_file}")
-            for (en_output_file, entry) in res:
+            for en_output_file, entry in res:
                 with HDF5File(self.configuration.output_file, "a") as master_file:
                     link_file = os.path.relpath(
                         en_output_file,
