@@ -43,6 +43,9 @@ H5FileKeys = namedtuple(
         "x_pixel_size",
         "y_pixel_size",
         "diode_keys",
+        "magnification",
+        "x_real_pixel_size",
+        "y_real_pixel_size",
     ],
 )
 
@@ -52,6 +55,7 @@ H5ScanTitles = namedtuple(
     [
         "init_titles",
         "init_zserie_titles",
+        "init_pcotomo_titles",
         "dark_titles",
         "flat_titles",
         "proj_titles",
