@@ -123,6 +123,8 @@ class TomoHDF5Config(ConfigBase):
 
     FIELD_OF_VIEW_DK = "field_of_view"
 
+    HANDLE_MACHINE_CURRENT = "create_control_data"
+
     COMMENTS_GENERAL_SECTION = {
         GENERAL_SECTION_DK: "general information. \n",
         INPUT_FILE_DK: "input file if not provided must be provided from the command line",
@@ -136,6 +138,7 @@ class TomoHDF5Config(ConfigBase):
         SINGLE_FILE_DK: "If True then will create a single file for all found sequences. "
         "If false create one nexus file per sequence and one master file with links to each sequence",
         FIELD_OF_VIEW_DK: "Force output to be a `Full` or a `Half` acquisition. If not provided we parse raw data to try to find this information.",
+        HANDLE_MACHINE_CURRENT: "Generate control/data (aka machine current). This part will need to interpolate from existing values and can take time in some cases.",
     }
 
     # KEYS SECTION
@@ -342,6 +345,7 @@ class TomoHDF5Config(ConfigBase):
         self._format = Format.STANDARD
         self._single_file = False
         self._bam_single_file = False
+        self._handle_machine_current = True
         # a single file is create by default if there is only one entry per file.
         # but we can enfore multi-file writing
 
@@ -371,6 +375,8 @@ class TomoHDF5Config(ConfigBase):
         self._alignment_titles = settings.Tomo.H5.ALIGNMENT_TITLES
         self._x_pixel_size_paths = settings.Tomo.H5.X_PIXEL_SIZE
         self._y_pixel_size_paths = settings.Tomo.H5.Y_PIXEL_SIZE
+        self._x_real_pixel_size_paths = settings.Tomo.H5.X_REAL_PIXEL_SIZE
+        self._y_real_pixel_size_paths = settings.Tomo.H5.Y_REAL_PIXEL_SIZE
         self._maginification_paths = settings.Tomo.H5.MAGNIFICATION_KEYS
 
         # information regarding frames types definition
@@ -484,6 +490,17 @@ class TomoHDF5Config(ConfigBase):
         else:
             self._bam_single_file = bam
 
+    @property
+    def handle_machine_current(self):
+        return self._handle_machine_current
+
+    @handle_machine_current.setter
+    def handle_machine_current(self, handle_machine_current: bool):
+        if not isinstance(handle_machine_current, bool):
+            raise TypeError("'handle_machine_current' should be a boolean")
+        else:
+            self._handle_machine_current = handle_machine_current
+
     # Keys section
 
     @property
@@ -697,6 +714,28 @@ class TomoHDF5Config(ConfigBase):
         else:
             self._y_pixel_size_paths = paths
 
+    @property
+    def x_real_pixel_size_paths(self) -> Iterable:
+        return self._x_real_pixel_size_paths
+
+    @x_real_pixel_size_paths.setter
+    def x_real_pixel_size_paths(self, paths):
+        if not isinstance(paths, Iterable):
+            raise TypeError("'paths should be an Iterable")
+        else:
+            self._x_real_pixel_size_paths = paths
+
+    @property
+    def y_real_pixel_size_paths(self) -> Iterable:
+        return self._y_real_pixel_size_paths
+
+    @y_real_pixel_size_paths.setter
+    def y_real_pixel_size_paths(self, paths):
+        if not isinstance(paths, Iterable):
+            raise TypeError("'paths should be an Iterable")
+        else:
+            self._y_real_pixel_size_paths = paths
+
     @property
     def magnification_keys(self) -> Iterable:
         return self._maginification_paths
@@ -903,6 +942,7 @@ class TomoHDF5Config(ConfigBase):
                 self.FIELD_OF_VIEW_DK: self.field_of_view.value
                 if self.field_of_view
                 else "",
+                self.HANDLE_MACHINE_CURRENT: self.handle_machine_current,
             },
             self.KEYS_SECTION_DK: {
                 TomoHDF5Config.VALID_CAMERA_DK: self.valid_camera_names or "",
@@ -1024,6 +1064,11 @@ class TomoHDF5Config(ConfigBase):
         single_file = dict_.get(TomoHDF5Config.SINGLE_FILE_DK, None)
         if single_file is not None:
             self.single_file = utils.convert_str_to_bool(single_file)
+        handle_machine_current = dict_.get(TomoHDF5Config.HANDLE_MACHINE_CURRENT, None)
+        if handle_machine_current is not None:
+            self.handle_machine_current = utils.convert_str_to_bool(
+                handle_machine_current
+            )
         input_format = dict_.get(TomoHDF5Config.INPUT_FORMAT_DK, None)
         if input_format is not None:
             if input_format == "":
@@ -1099,15 +1144,15 @@ class TomoHDF5Config(ConfigBase):
                 y_pixel_size_paths, none_if_empty=True
             )
             self.y_pixel_size_paths = y_pixel_size_paths
-        # handle sample detector distance
-        sample_detector_distance = dict_.get(
+        # handle sample detector distance paths
+        sample_detector_distance_paths = dict_.get(
             TomoHDF5Config.SAMPLE_DETECTOR_DISTANCE_DK, None
         )
-        if sample_detector_distance is not None:
-            sample_detector_distance = utils.convert_str_to_tuple(
-                sample_detector_distance, none_if_empty=True
+        if sample_detector_distance_paths is not None:
+            sample_detector_distance_paths = utils.convert_str_to_tuple(
+                sample_detector_distance_paths, none_if_empty=True
             )
-            self.sample_detector_distance_paths = sample_detector_distance
+            self.sample_detector_distance_paths = sample_detector_distance_paths
 
     def load_entries_titles_section(self, dict_):
         # handle entries to convert
@@ -1242,7 +1287,6 @@ class TomoHDF5Config(ConfigBase):
 
 
 class XRD3DHDF5Config(TomoHDF5Config):
-
     ROCKING_KEYS_DK = "ROCKING_KEYS"
 
     TomoHDF5Config.COMMENTS.update(
