@@ -38,6 +38,7 @@ __date__ = "10/03/2021"
 
 from typing import Union
 from nxtomomill.io.framegroup import FrameGroup
+from silx.utils.enum import Enum as _Enum
 import re
 import logging
 
@@ -93,6 +94,17 @@ def convert_str_to_tuple(
         return tuple(elmts)
 
 
+def convert_str_to_bool(value: Union[str, bool]):
+    if isinstance(value, bool):
+        return value
+    elif isinstance(value, str):
+        if value not in ("False", "True"):
+            raise ValueError("value should be 'True' or 'False'")
+        return value == "True"
+    else:
+        raise TypeError("value should be a string")
+
+
 def is_url_path(url_str: str) -> bool:
     """
     :return: True if the provided string fit DataUrl pattern
@@ -144,3 +156,8 @@ def convert_str_to_frame_grp(input_str: str) -> tuple:
             result.append(frame_grp)
 
     return tuple(result)
+
+
+class PathType(_Enum):
+    ABSOLUTE = "absolute"
+    RELATIVE = "relative"
