@@ -29,6 +29,8 @@ __license__ = "MIT"
 __date__ = "10/02/2022"
 
 
+from functools import partial
+from operator import is_not
 from silx.utils.proxy import docstring
 from typing import Optional
 from nxtomomill.nexus.nxdetector import NXdetector, NXdetectorWithUnit
@@ -39,6 +41,9 @@ from tomoscan.unitsystem.voltagesystem import VoltageSystem
 from .utils import get_data
 from tomoscan.io import HDF5File
 from .nxobject import NXobject
+import logging
+
+_logger = logging.getLogger(__name__)
 
 
 class NXinstrument(NXobject):
@@ -63,16 +68,6 @@ class NXinstrument(NXobject):
         self._name = None
         self._set_freeze(True)
 
-    @property
-    def name(self) -> Optional[str]:
-        return self._name
-
-    @name.setter
-    def name(self, name: Optional[str]):
-        if not isinstance(name, (str, type(None))):
-            raise TypeError(f"name is expected to be None or a str. Not {type(name)}")
-        self._name = name
-
     @property
     def detector(self) -> Optional[NXdetector]:
         return self._detector
@@ -205,3 +200,42 @@ class NXinstrument(NXobject):
                 file_path=file_path,
                 data_path="/".join([data_path, nexus_instrument_paths.NAME]),
             )
+
+    @staticmethod
+    @docstring(NXobject)
+    def concatenate(nx_objects: tuple, node_name="instrument"):
+        # filter None obj
+        nx_objects = tuple(filter(partial(is_not, None), nx_objects))
+        if len(nx_objects) == 0:
+            return None
+        # warning: later we make the assumption that nx_objects contains at least one element
+        for nx_obj in nx_objects:
+            if not isinstance(nx_obj, NXinstrument):
+                raise TypeError("Cannot concatenate non NXinstrument object")
+
+        nx_instrument = NXinstrument(node_name=node_name)
+
+        nx_instrument.name = nx_objects[0].name
+        _logger.info(f"instrument name {nx_objects[0].name} will be picked")
+
+        nx_instrument.source = NXsource.concatenate(
+            [nx_obj.source for nx_obj in nx_objects],
+            node_name="source",
+        )
+        nx_instrument.source.parent = nx_instrument
+
+        nx_instrument.diode = NXdetectorWithUnit.concatenate(
+            [nx_obj.diode for nx_obj in nx_objects],
+            node_name="diode",
+            expected_dim=(1,),
+            default_unit=VoltageSystem.VOLT,
+        )
+        nx_instrument.diode.parent = nx_instrument
+
+        nx_instrument.detector = NXdetector.concatenate(
+            [nx_obj.detector for nx_obj in nx_objects],
+            node_name="detector",
+        )
+        nx_instrument.detector.parent = nx_instrument
+
+        return nx_instrument
