@@ -31,11 +31,16 @@ __date__ = "03/02/2022"
 
 from typing import Optional
 from silx.io.url import DataUrl
+from tomoscan.esrf.scan.utils import cwd_context
 from tomoscan.unitsystem import Unit
+from tomoscan.io import HDF5File
 from tomoscan.nexus.paths.nxtomo import LATEST_VERSION as LATEST_NXTOMO_VERSION
 from silx.io.dictdump import dicttonx
 import h5py
 import os
+from nxtomomill.utils.file_path import to_target_rel_path
+
+from nxtomomill.utils.h5pyutils import from_data_url_to_virtual_source
 
 
 class ElementWithUnit:
@@ -84,7 +89,6 @@ class ElementWithUnit:
 
 
 class NXobject:
-
     __isfrozen = False
     # to ease API and avoid setting wrong attributes we 'freeze' the attributes
     # see https://stackoverflow.com/questions/3603502/prevent-creating-new-attributes-outside-init
@@ -171,10 +175,14 @@ class NXobject:
             raise TypeError
 
         entry_path = "/".join([data_path, self.path])
+        if entry_path in ("/", ""):
+            raise ValueError(
+                "data_path AND nx_object cannot be both empty strings (you must provide a name to nx tomo"
+            )
         # not fully sure about the dicttoh5 "add" behavior
         if os.path.exists(file_path):
             with h5py.File(file_path, mode="a") as h5f:
-                if entry_path in h5f:
+                if entry_path != "/" and entry_path in h5f:
                     if overwrite:
                         del h5f[entry_path]
                     else:
@@ -191,6 +199,7 @@ class NXobject:
             datasets_to_handle_in_postprocessing[key] = nx_dict.pop(key)
         for key in self._get_data_urls(nx_dict):
             datasets_to_handle_in_postprocessing[key] = nx_dict.pop(key)
+        master_vds_file = self._get_vds_master_file_folder(nx_dict)
 
         # retrieve attributes
         attributes = {}
@@ -210,35 +219,75 @@ class NXobject:
             update_mode="replace",
             mode="a",
         )
-        assert os.path.exists(file_path)
-        # now handle nx_dict containing h5py.virtualSource or DataUrl
-        # this cannot be handled from the nxdetector class because not aware about
-        # the output file.
-        for (
-            dataset_path,
-            v_sources_or_data_urls,
-        ) in datasets_to_handle_in_postprocessing.items():
-            for v_source_or_data_url in v_sources_or_data_urls:
-                if isinstance(v_source_or_data_url, DataUrl):
-                    data = DataUrl(
-                        file_path=v_source_or_data_url.file_path(),
-                        data_path=v_source_or_data_url.data_path(),
-                        scheme=v_source_or_data_url.scheme(),
-                        data_slice=v_source_or_data_url.data_slice(),
-                    )
-                else:
-                    data = v_source_or_data_url
-                from nxtomomill.utils.frameappender import (
-                    FrameAppender,
-                )  # avoid cyclic import
 
-                frame_appender = FrameAppender(
-                    data=data,
-                    file_path=file_path,
-                    data_path="/".join([data_path, dataset_path]),
-                    where="end",
-                )
-                frame_appender.process()
+        assert os.path.exists(file_path)
+
+        # in order to solve relative path we need to be on the (source) master file working directory
+        with cwd_context(master_vds_file):
+            # now handle nx_dict containing h5py.virtualSource or DataUrl
+            # this cannot be handled from the nxdetector class because not aware about
+            # the output file.
+            for (
+                dataset_path,
+                v_sources_or_data_urls,
+            ) in datasets_to_handle_in_postprocessing.items():
+                data_type = None
+                vs_shape = None
+                n_frames = 0
+
+                v_sources_to_handle_in_postprocessing = []
+                # convert DataUrl to VirtualSource
+                dataset_keys = v_sources_or_data_urls
+                for v_source_or_data_url in dataset_keys:
+                    if isinstance(v_source_or_data_url, DataUrl):
+                        vs = from_data_url_to_virtual_source(v_source_or_data_url)[0]
+                    else:
+                        assert isinstance(
+                            v_source_or_data_url, h5py.VirtualSource
+                        ), "v_source_or_data_url is not a DataUrl or a VirtualSource"
+                        vs = v_source_or_data_url
+
+                    if data_type is None:
+                        data_type = vs.dtype
+                    elif vs.dtype != data_type:
+                        raise TypeError(
+                            f"Virtual sources have incoherent data types (found {data_type} and {vs.dtype})"
+                        )
+
+                    if not len(vs.maxshape) == 3:
+                        raise ValueError(
+                            f"Virtual sources are expected to be 3D. {len(vs.maxshape)} found"
+                        )
+                    if vs_shape is None:
+                        vs_shape = vs.maxshape[1:]
+                    elif vs_shape != vs.maxshape[1:]:
+                        raise ValueError(
+                            f"Virtual sources are expected to have same frame dimensions. found {vs_shape} and {vs.maxshape[1:]}"
+                        )
+                    n_frames += vs.maxshape[0]
+                    vs.path = to_target_rel_path(vs.path, file_path)
+                    v_sources_to_handle_in_postprocessing.append(vs)
+
+                if n_frames == 0:
+                    # in the case there is no frame to be saved
+                    return
+
+                vs_shape = [
+                    n_frames,
+                ] + list(vs_shape)
+                layout = h5py.VirtualLayout(shape=tuple(vs_shape), dtype=data_type)
+                # fill virtual dataset
+                loc_pointer = 0
+                for v_source in v_sources_to_handle_in_postprocessing:
+                    layout[
+                        loc_pointer : (loc_pointer + v_source.maxshape[0])
+                    ] = v_source
+                    loc_pointer += v_source.maxshape[0]
+
+                with HDF5File(file_path, mode="a") as h5s:
+                    h5s.create_virtual_dataset(
+                        "/".join([data_path, dataset_path]), layout
+                    )
 
         # write attributes of dataset defined from a list of DataUrl or VirtualSource
         assert os.path.exists(file_path)
@@ -263,8 +312,8 @@ class NXobject:
         """
         raise NotImplementedError("Base class")
 
-    def __str__(self):
-        f"{type(self)}: {self.path}"
+    def __str__(self) -> str:
+        return f"{type(self)}: {self.path}"
 
     @staticmethod
     def _get_virtual_sources(ddict) -> tuple:
@@ -285,6 +334,14 @@ class NXobject:
                 keys.append(key)
         return tuple(keys)
 
+    @staticmethod
+    def _get_vds_master_file_folder(nx_dict: dict):
+        path = nx_dict.pop("__vds_master_file__", None)
+        if path is not None:
+            return os.path.dirname(path)
+        else:
+            return None
+
     @staticmethod
     def _get_data_urls(ddict) -> tuple:
         """Return key / path containing a list or a tuple of silx.io.url.DataUrl"""
@@ -309,3 +366,12 @@ class NXobject:
             raise AttributeError("can't set attribute", __name)
         else:
             super().__setattr__(__name, __value)
+
+    @staticmethod
+    def concatenate(nx_objects: tuple, node_name: str):
+        """
+        concatenate a tuple of NXobject into a single NXobject
+        :param Iterable Nx-objects: nx object to concatenate
+        :param str node_name: name of the node to create. Parent must be handled manually for now.
+        """
+        raise NotImplementedError("Base class")
