@@ -29,6 +29,8 @@ __license__ = "MIT"
 __date__ = "03/02/2022"
 
 
+from functools import partial
+from operator import is_not
 from typing import Iterable, Optional
 
 import numpy
@@ -37,6 +39,9 @@ from .utils import cast_and_check_array_1D, get_data_and_unit, get_data
 from silx.utils.proxy import docstring
 from tomoscan.nexus.paths.nxtomo import get_paths as get_nexus_paths
 from tomoscan.unitsystem.metricsystem import MetricSystem
+import logging
+
+_logger = logging.getLogger(__name__)
 
 
 class NXsample(NXobject):
@@ -59,8 +64,6 @@ class NXsample(NXobject):
 
     @name.setter
     def name(self, name: Optional[str]) -> None:
-        if isinstance(name, numpy.ndarray):
-            name = numpy.array2string(name)
         if not isinstance(name, (type(None), str)):
             raise TypeError(f"name is expected to be None or str not {type(name)}")
         self._name = name
@@ -109,7 +112,6 @@ class NXsample(NXobject):
         nexus_path_version: Optional[float] = None,
         data_path: Optional[str] = None,
     ) -> dict:
-
         nexus_paths = get_nexus_paths(nexus_path_version)
         nexus_sample_paths = nexus_paths.nx_sample_paths
 
@@ -121,7 +123,7 @@ class NXsample(NXobject):
         if self.rotation_angle is not None:
             path_rotation_angle = f"{self.path}/{nexus_sample_paths.ROTATION_ANGLE}"
             nx_dict[path_rotation_angle] = self.rotation_angle
-            nx_dict["@".join([path_rotation_angle, "unit"])] = "degree"
+            nx_dict["@".join([path_rotation_angle, "units"])] = "degree"
         if self.rocking is not None:
             path_rocking = f"{self.path}/{nexus_sample_paths.ROCKING}"
             nx_dict[path_rocking] = self.rocking
@@ -134,19 +136,19 @@ class NXsample(NXobject):
         if self.x_translation.value is not None:
             path_x_translation = f"{self.path}/{nexus_sample_paths.X_TRANSLATION}"
             nx_dict[path_x_translation] = self.x_translation.value
-            nx_dict["@".join([path_x_translation, "unit"])] = str(
+            nx_dict["@".join([path_x_translation, "units"])] = str(
                 self.x_translation.unit
             )
         if self.y_translation.value is not None:
             path_y_translation = f"{self.path}/{nexus_sample_paths.Y_TRANSLATION}"
             nx_dict[path_y_translation] = self.y_translation.value
-            nx_dict["@".join([path_y_translation, "unit"])] = str(
+            nx_dict["@".join([path_y_translation, "units"])] = str(
                 self.y_translation.unit
             )
         if self.z_translation.value is not None:
             path_z_translation = f"{self.path}/{nexus_sample_paths.Z_TRANSLATION}"
             nx_dict[path_z_translation] = self.z_translation.value
-            nx_dict["@".join([path_z_translation, "unit"])] = str(
+            nx_dict["@".join([path_z_translation, "units"])] = str(
                 self.z_translation.unit
             )
 
@@ -192,3 +194,62 @@ class NXsample(NXobject):
             data_path="/".join([data_path, nexus_sample_paths.Z_TRANSLATION]),
             default_unit=MetricSystem.METER,
         )
+
+    @staticmethod
+    @docstring(NXobject)
+    def concatenate(nx_objects: tuple, node_name="sample"):
+        nx_objects = tuple(filter(partial(is_not, None), nx_objects))
+        # filter None obj
+        if len(nx_objects) == 0:
+            return None
+        # warning: later we make the assumption that nx_objects contains at least one element
+        for nx_obj in nx_objects:
+            if not isinstance(nx_obj, NXsample):
+                raise TypeError("Cannot concatenate non NXsample object")
+
+        nx_sample = NXsample(node_name)
+        _logger.info(f"sample name {nx_objects[0].name} will be picked")
+        nx_sample.name = nx_objects[0].name
+
+        rotation_angles = [
+            nx_obj.rotation_angle
+            for nx_obj in nx_objects
+            if nx_obj.rotation_angle is not None
+        ]
+        if len(rotation_angles) > 0:
+            nx_sample.rotation_angle = numpy.concatenate(rotation_angles)
+
+        x_translations = [
+            nx_obj.x_translation.value * nx_obj.x_translation.unit.value
+            for nx_obj in nx_objects
+            if nx_obj.x_translation is not None
+        ]
+        if len(x_translations) > 0:
+            nx_sample.x_translation = numpy.concatenate(x_translations)
+
+        y_translations = [
+            nx_obj.y_translation.value * nx_obj.y_translation.unit.value
+            for nx_obj in nx_objects
+            if nx_obj.y_translation.value is not None
+        ]
+        if len(y_translations) > 0:
+            nx_sample.y_translation = numpy.concatenate(y_translations)
+
+        z_translations = [
+            nx_obj.z_translation.value * nx_obj.z_translation.unit.value
+            for nx_obj in nx_objects
+            if nx_obj.z_translation.value is not None
+        ]
+        if len(z_translations) > 0:
+            nx_sample.z_translation = numpy.concatenate(z_translations)
+
+        rocking_list = list(
+            filter(
+                partial(is_not, None),
+                [nx_obj.rocking for nx_obj in nx_objects],
+            )
+        )
+        if len(rocking_list) > 0:
+            nx_sample.rocking = numpy.concatenate(rocking_list)
+
+        return nx_sample
