@@ -123,6 +123,9 @@ def get_data_and_unit(file_path, data_path, default_unit):
                 unit = dataset.attrs["units"]
             else:
                 unit = default_unit
+            if hasattr(unit, "decode"):
+                # handle Diamond dataset
+                unit = unit.decode()
             return h5py_read_dataset(dataset), unit
         else:
             return None, default_unit
@@ -136,7 +139,7 @@ def get_data(file_path, data_path):
             return None
 
 
-def concatenate(nx_objects, **kwargs):
+def concatenate(nx_objects: Iterable, **kwargs):
     if len(nx_objects) == 0:
         return None
     else:
