@@ -35,6 +35,8 @@ from importlib.machinery import SourceFileLoader
 import logging
 import inspect
 
+from nxtomomill.nexus.nxtomo import NXtomo
+
 _logger = logging.getLogger(__name__)
 
 
@@ -80,8 +82,10 @@ def get_plugins_instances(directory: str):
         ):
             full_path = os.path.join(directory, file_)
             try:
-                mod = SourceFileLoader(file_, full_path).load_module()
-            except:
+                mod = SourceFileLoader(  # pylint: disable=E1120
+                    file_, full_path
+                ).load_module()
+            except Exception:
                 _logger.warning("Fail to import " + full_path)
             else:
                 plugins.extend(_load_plugin_from_module(mod))
@@ -150,22 +154,6 @@ class HDF5Plugin(_PluginBase):
     :param positioner_keys: which information you will need for the plugin.
     :param instrument_keys: which information to be read from the "instrument"
                             dataset of the input bliss scan.
-    :param sample_datasets_created: name of the field created by the plugin in
-                                    the sample group. Allows those to be skip
-                                    by nxtomomill and ignore some warnings.
-    :type sample_datasets_created: Union[None, Iterable]
-    :param detector_datasets_created: name of the field created by the plugin
-                                      in the detector group. Allows those to be
-                                      skip by nxtomomill and ignore some
-                                      warnings.
-    :type detector_datasets_created: Union[None, Iterable]
-    :param beam_datasets_created: name of the field created by the plugin in
-                                  the beam group. Allows those to be skip by
-                                  nxtomomill and ignore some warnings.
-    :type beam_datasets_created: Union[None, Iterable]
-    :warning: unit are not managed by the plugin. So you will retrieve
-    raw data and should know the unit of the data you are reading
-
     """
 
     def __init__(
@@ -173,9 +161,6 @@ class HDF5Plugin(_PluginBase):
         name,
         positioner_keys=tuple(),
         instrument_keys=tuple(),
-        sample_datasets_created=None,
-        detector_datasets_created=None,
-        beam_datasets_created=None,
     ):
         super().__init__(name)
         """As the sequence is splitted into several scan we need to parse
@@ -194,9 +179,6 @@ class HDF5Plugin(_PluginBase):
             if not isinstance(key, str):
                 raise TypeError("resource names should be strings")
             self.__instrument_info[key] = None
-        self._sample_datasets_created = sample_datasets_created
-        self._detector_datasets_created = detector_datasets_created
-        self._beam_datasets_created = beam_datasets_created
 
     def process(self) -> list:
         """
@@ -243,14 +225,5 @@ class HDF5Plugin(_PluginBase):
         """
         return self.__instrument_info[name]
 
-    @property
-    def sample_datasets_created(self):
-        return self._sample_datasets_created
-
-    @property
-    def detector_datasets_created(self):
-        return self._detector_datasets_created
-
-    @property
-    def beam_datasets_created(self):
-        return self._beam_datasets_created
+    def update_nx_tomo(self, nx_tomo: NXtomo):
+        raise NotImplementedError("Base class")
