@@ -104,7 +104,7 @@ class Tomo:
         """if a scan starts by one of those string then is considered as
         alignment scan"""
 
-        X_PIXEL_SIZE = (
+        X_PIXEL_SIZE = (  # aka "sample" pixel size
             "technique/optic/sample_pixel_size",
             "technique/detector/pixel_size",
             "hry_step_size",
@@ -112,12 +112,22 @@ class Tomo:
         """Possible path to th pixel size."""
         # warning: we can have two cases: one with an empty space at the end or not
 
-        Y_PIXEL_SIZE = (
+        Y_PIXEL_SIZE = (  # aka "sample" pixel size
             "technique/optic/sample_pixel_size",
             "technique/detector/pixel_size",
             "hry_step_size",
         )
 
+        X_REAL_PIXEL_SIZE = (
+            "technique/detector/{detector_name}/pixel_size",
+            "technique/detector/pixel_size",
+        )
+
+        Y_REAL_PIXEL_SIZE = (
+            "technique/detector/{detector_name}/pixel_size",
+            "technique/detector/pixel_size",
+        )
+
         MAGNIFICATION_KEYS = ("technique/optic/magnification",)
 
         DISTANCE_KEYS = ("technique/scan/sample_detector_distance",)
@@ -156,7 +166,6 @@ class Tomo:
 
 class XRD3D(Tomo):
     class H5(Tomo.H5):
-
         positioners_path = "instrument/positioners/"
 
         ROT_ANGLE_KEYS = (positioners_path + "diffrz", positioners_path + "rot")
