@@ -52,7 +52,6 @@ def from_data_url_to_virtual_source(url: DataUrl) -> tuple:
         )
 
     with HDF5File(url.file_path(), mode="r") as o_h5s:
-
         original_data_shape = o_h5s[url.data_path()].shape
         data_type = o_h5s[url.data_path()].dtype
         if len(original_data_shape) == 2:
@@ -70,7 +69,9 @@ def from_data_url_to_virtual_source(url: DataUrl) -> tuple:
                 original_data_shape[-1],
             )
 
-    vs = h5py.VirtualSource(url.file_path(), url.data_path(), shape=vs_shape)
+    vs = h5py.VirtualSource(
+        url.file_path(), url.data_path(), shape=vs_shape, dtype=data_type
+    )
 
     if url.data_slice() is not None:
         vs.sel = selection.select(original_data_shape, url.data_slice())
