@@ -42,7 +42,7 @@ from silx.io.url import DataUrl
 class _BaseReader(contextlib.AbstractContextManager):
     def __init__(self, url: DataUrl):
         if not isinstance(url, DataUrl):
-            raise TypeError("url should be an instance of DataUrl")
+            raise TypeError(f"url should be an instance of DataUrl. Not {type(url)}")
         if url.scheme() not in ("silx", "h5py"):
             raise ValueError("Valid scheme are silx and h5py")
         if url.data_slice() is not None:
