@@ -30,6 +30,7 @@ __date__ = "29/04/2019"
 
 
 import typing
+from datetime import datetime
 import numpy
 import os
 from tomoscan.io import HDF5File
@@ -37,7 +38,7 @@ from silx.io.url import DataUrl
 from silx.io.utils import get_data
 import logging
 from silx.utils.deprecation import deprecated
-from .context import cwd_context
+from tomoscan.esrf.scan.utils import cwd_context
 from .frameappender import FrameAppender
 from silx.utils.enum import Enum as _Enum
 
@@ -105,7 +106,9 @@ def get_file_name(file_name, extension, check=True):
     :param bool check: if check, already check if the file as one of the
                        '_FileExtension'
     """
-    extension = FileExtension.from_value(extension.lower())
+    if isinstance(extension, str):
+        extension = FileExtension.from_value(extension.lower())
+    assert isinstance(extension, FileExtension)
     if check:
         for value in FileExtension.values():
             if file_name.lower().endswith(value):
@@ -289,12 +292,14 @@ def add_dark_flat_nx_file(
         x_translation_path = os.path.join(entry, "sample", "x_translation")
         y_translation_path = os.path.join(entry, "sample", "y_translation")
         z_translation_path = os.path.join(entry, "sample", "z_translation")
+        control_data_path = os.path.join(entry, "control", "data")
         data_key_paths = (
             count_time_path,
             rotation_angle_path,
             x_translation_path,
             y_translation_path,
             z_translation_path,
+            control_data_path,
         )
         mandatory_keys = (
             "count_time",
@@ -304,6 +309,7 @@ def add_dark_flat_nx_file(
             "x_translation",
             "y_translation",
             "z_translation",
+            "control/data",
         )
 
         data_keys = tuple(list(mandatory_keys) + list(optional_keys))
@@ -459,3 +465,33 @@ def change_image_key_control(
             image_key_value = ImageKey.PROJECTION
         image_keys[frames_indexes] = image_key_value.value
         node[image_keys_path][:] = image_keys
+
+
+def str_datetime_to_numpy_datetime64(
+    my_datetime: typing.Union[str, datetime]
+) -> numpy.datetime64:
+    # numpy deprecates time zone awarness conversion to numpy.datetime64.
+    # so we remove the time zone info.
+    if isinstance(my_datetime, str):
+        datetime_as_datetime = datetime.fromisoformat(my_datetime)
+    elif isinstance(my_datetime, datetime):
+        datetime_as_datetime = my_datetime
+    else:
+        raise TypeError(
+            f"my_datetime is expected to be a str or an instance of datetime. Not {type(my_datetime)}"
+        )
+
+    datetime_as_utc_datetime = datetime_as_datetime.astimezone(None)
+    tz_free_datetime_as_datetime = datetime_as_utc_datetime.replace(tzinfo=None)
+    return numpy.datetime64(tz_free_datetime_as_datetime).astype("<M8[ms]")
+
+
+def strip_extension(filename, logger=None):
+    if filename.endswith((".nx", ".h5")):
+        return filename[:-3]
+    elif filename.endswith(".hdf5"):
+        return filename[:-5]
+    else:
+        if logger is not None:
+            logger.warning(f"Unusual file name {filename} has no known postfix")
+        return filename
